﻿// ECMAScript 5 strict mode
"use strict";

assert2(cr, "cr namespace not created");
assert2(cr.plugins_, "cr.plugins_ not created");

/////////////////////////////////////
// Plugin class
cr.plugins_.njrc4 = function(runtime)
{
	this.runtime = runtime;
};

(function ()
{
	/////////////////////////////////////
	var pluginProto = cr.plugins_.njrc4.prototype;
		
	/////////////////////////////////////
	// Object type class
	pluginProto.Type = function(plugin)
	{
		this.plugin = plugin;
		this.runtime = plugin.runtime;
	};

	var typeProto = pluginProto.Type.prototype;

	// called on startup for each object type
	typeProto.onCreate = function()
	{

		
	};

	/////////////////////////////////////
	// Instance class
	pluginProto.Instance = function(type)
	{
		this.type = type;
		this.runtime = type.runtime;
	};
	
	var instanceProto = pluginProto.Instance.prototype;

	// called whenever an instance is created
	instanceProto.onCreate = function()
	{
		this.asciii = new Array(256);

		for(var i=0;i<256;i++) {
			this.asciii[i] = String.fromCharCode(i);
		}
		
	};
	
	instanceProto.chr = function (num)
	{
		return this.asciii[num];
	}
	
	instanceProto.ord = function (txt)
	{
		for (var i = 0; i < 256; i++)
		{
			if (this.asciii[i] === txt)
			{
				return i;
			}
		}
		return -1;
	}	
	//////////////////////////////////////
	// Conditions
	function Cnds() {};
	
	pluginProto.cnds = new Cnds();
	
	//////////////////////////////////////
	// Actions
	function Acts() {};
	
	pluginProto.acts = new Acts();
	
	//////////////////////////////////////
	// Expressions
	function Exps() {};
	

	Exps.prototype.Crypt = function (ret, akey, astr, amode)
	{
		function rc4(key, str) {
			var s = [],
				j = 0,
				x, res = '';
			for (var i = 0; i < 256; i++) {
				s[i] = i;
			}
			for (i = 0; i < 256; i++) {
				j = (j + s[i] + key.charCodeAt(i % key.length)) % 256;
				x = s[i];
				s[i] = s[j];
				s[j] = x;
			}
			i = 0;
			j = 0;
			for (var y = 0; y < str.length; y++) {
				i = (i+1) % 256;
				j = (j + s[i]) % 256;
				x = s[i];
				s[i] = s[j];
				s[j] = x;
				res += String.fromCharCode(str.charCodeAt(y) ^ s[(s[i]+s[j]) % 256]);
			}
			return res;
		}
		
		function rc4_mod_256(key, str) {
			var s = [],
				j = 0,
				x, res = '';
			for (var i = 0; i < 256; i++) {
				s[i] = i;
			}
			for (i = 0; i < 256; i++) {
				j = (j + s[i] + key[(i % key.length)]) % 256;
				x = s[i];
				s[i] = s[j];
				s[j] = x;
			}
			i = 0;
			j = 0;
			var strlen = str.length;
			for (var y = 0; y < strlen; y++) {
				i = (i+1) % 256;
				j = (j + s[i]) % 256;
				x = s[i];
				s[i] = s[j];
				s[j] = x;
				str[y] = str[y] ^ s[(s[i] + s[j]) % 256];
			}
		}
		ret.set_string(rc4(akey,astr));
		
	};	
	
	Exps.prototype.GetAsciiTable = function (ret)
	{
		var mtxt = "";
		for(var i=0;i<256;i++) {
			mtxt += String.fromCharCode(i)+",";
		}

		ret.set_string(mtxt);
	};		

	Exps.prototype.GetAsciiChar = function (ret,code)
	{
		ret.set_string(String.fromCharCode(code));
	};	

	Exps.prototype.Encrypt = function (ret,pass,algo,txt)
	{
		var crypto = require("crypto"),
		algorithm = algo,
		password = pass;
		
		function encrypt(text){
			var cipher = crypto.createCipher(algorithm,password);
			var crypted = cipher.update(text,"utf8","hex");
			crypted += cipher.final("hex");
			return crypted;
		}
		
		var hw = encrypt(txt);
		
		ret.set_string(hw);
	};		
	
	Exps.prototype.EncryptB64 = function (ret,pass,algo,txt)
	{
		var crypto = require("crypto"),
		algorithm = algo,
		password = pass;
		
		function encrypt(text){
			var cipher = crypto.createCipher(algorithm,password);
			var crypted = cipher.update(text,"utf8","base64");
			crypted += cipher.final("base64");
			return crypted;
		}
		
		var hw = encrypt(txt);
		
		ret.set_string(hw);
	};	
	
	Exps.prototype.Decrypt = function (ret,pass,algo,txt)
	{
		var crypto = require("crypto"),
		algorithm = algo,
		password = pass;
		
		function decrypt(text){
			var decipher = crypto.createDecipher(algorithm,password);
			var dec = decipher.update(text,"hex","utf8");
			dec += decipher.final("utf8");
			return dec;
		}
		
		var hh = decrypt(txt);
		
		ret.set_string(hh);
	};		
	
	Exps.prototype.DecryptB64 = function (ret,pass,algo,txt)
	{
		var crypto = require("crypto"),
		algorithm = algo,
		password = pass;
		
		function decrypt(text){
			var decipher = crypto.createDecipher(algorithm,password);
			var dec = decipher.update(text,"base64","utf8");
			dec += decipher.final("utf8");
			return dec;
		}
		
		var hh = decrypt(txt);
		
		ret.set_string(hh);
	};	
	
	pluginProto.exps = new Exps();

}());